/*
Copyright 2022, Savanni D'Gerinel <savanni@luminescent-dreams.com>

This file is part of Savanni's AVR library.

Lumeto is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

Lumeto is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with Lumeto. If not, see <https://www.gnu.org/licenses/>.
*/

#include <util/delay.h>
#include <base.h>
#include <shift_register.h>

void sr_strobe_line(gpio_t *line) {
    set_line(line);
    _delay_us(100);
    clear_line(line);
}

void sr_initialize(shift_register_t *reg) {
    set_line_direction(&reg->output, LINE_OUT);
    set_line_direction(&reg->shift_clock, LINE_OUT);
    set_line_direction(&reg->latch_clock, LINE_OUT);

    clear_line(&reg->output);
    clear_line(&reg->shift_clock);
    clear_line(&reg->latch_clock);
}

void sr_send_lsb(shift_register_t *reg, uint8_t byte) {
    for (uint8_t i = 0; i < 8; i++) {
        byte & _BV(i) ? set_line(&reg->output) : clear_line(&reg->output);
        sr_strobe_line(&reg->shift_clock);
    }
    clear_line(&reg->output);
    sr_strobe_line(&reg->latch_clock);
}

void sr_send_msb(shift_register_t *reg, uint8_t byte) {
    for (int8_t i = 7; i >= 0; i--) {
        byte & _BV(i) ? set_line(&reg->output) : clear_line(&reg->output);
        sr_strobe_line(&reg->shift_clock);
    }
    clear_line(&reg->output);
    sr_strobe_line(&reg->latch_clock);
}


