/*
Copyright 2022, Savanni D'Gerinel <savanni@luminescent-dreams.com>

This file is part of Savanni's AVR library.

Lumeto is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

Lumeto is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with Lumeto. If not, see <https://www.gnu.org/licenses/>.
*/


#include <spi.h>
#include <util/delay.h>

void spi_initialize(spi_t *spi) {
    set_line_direction(&spi->clock, LINE_OUT);
    set_line_direction(&spi->data_out, LINE_OUT);
    set_line_direction(&spi->data_in, LINE_IN);
    set_line_direction(&spi->chip_select, LINE_OUT);

    set_line(&spi->chip_select);
}

void spi_acquire(spi_t *spi) {
    clear_line(&spi->chip_select);
}

void spi_release(spi_t *spi) {
    set_line(&spi->chip_select);
}

uint8_t spi_transfer_byte(spi_t *spi, uint8_t output) {
    uint8_t input = 0;
    int input_bit;

    for (int i = 7; i >= 0; i--) {
        clear_line(&spi->clock);
        if (output & _BV(i)) {
            set_line(&spi->data_out);
        } else {
            clear_line(&spi->data_out);
        }
        _delay_us(10);
        set_line(&spi->clock);
        _delay_us(10);
        input_bit = read_line(&spi->data_in);
        input = (input << 1) | input_bit;
    }
    clear_line(&spi->clock);
    _delay_us(10);
    return input;
}

/*
void send_bytes(spi_t *line, u8 *byte, size_t length, u8 *input, size_t input_length) {

}
*/

